package cz.novoj.hugeImport.aop;

import cz.novoj.hugeImport.AbstractImportLogic;
import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.jdbc.core.JdbcTemplate;

/**
 * Description
 *
 * @author Jan Novotn, FG Forrest a.s. (c) 2007
 * @version $Id: $
 */
public class LockingAdvice implements MethodInterceptor {
	private static final Log log = LogFactory.getLog(LockingAdvice.class);
	private JdbcTemplate jdbcTemplate;

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}

	public Object invoke(MethodInvocation methodInvocation) throws Throwable {
		try {
			AbstractImportLogic importLogic = (AbstractImportLogic)methodInvocation.getThis();
			String dbEngine = importLogic.getImplementation();
			doLock("T_RECIPIENT_" + dbEngine, "T_RECIPIENT_PROPS_" + dbEngine);
			return methodInvocation.proceed();
		} finally {
			doUnlock();
		}
	}

	private void doLock(String... tableName) {
		StringBuilder sql = new StringBuilder("LOCK TABLES ");
		for(int i = 0; i < tableName.length; i++) {
			String name = tableName[i];
			sql.append(name);
			if (i + 1 < tableName.length) {
				sql.append(" WRITE, ");
			} else {
				sql.append(" WRITE");
			}
		}
		log.info("SQL: " + sql);
		jdbcTemplate.execute(sql.toString());
	}

	private void doUnlock() {
		log.info("SQL: UNLOCK TABLES");
		jdbcTemplate.execute("UNLOCK TABLES");
	}

}
