package cz.novoj.hugeImport;

import cz.novoj.spring.utils.HostConfigurableContextLoader;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.io.ClassPathResource;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.jdbc.SimpleJdbcTestUtils;

import javax.sql.DataSource;

/**
 * Description
 *
 * @author Jan Novotn, FG Forrest a.s. (c) 2007
 * @version $Id: $
 */
@SuppressWarnings({"JUnitTestMethodWithNoAssertions"})
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(
		locations = {
				"classpath:spring/datasource.xml",
				"classpath:spring/main.xml"
				},
		loader = HostConfigurableContextLoader.class
)
public abstract class AbstractImportLogicTest {
	protected static final Log log = LogFactory.getLog(AbstractImportLogicTest.class);
	protected AbstractImportLogic importLogic;
	@Autowired @Qualifier("dataSource")	private DataSource dataSource;
	private static final int LONG_CHUNK_SIZE = 10000;
	private static final int SHORT_CHUNK_SIZE = 2000;

	@Before
	public void initializeDb() {
		SimpleJdbcTemplate simpleJdbcTemplate = new SimpleJdbcTemplate(dataSource);
		SimpleJdbcTestUtils.executeSqlScript(simpleJdbcTemplate, new ClassPathResource("/db/drop.sql"), false);
		SimpleJdbcTestUtils.executeSqlScript(simpleJdbcTemplate, new ClassPathResource("/db/create.sql"), false);
	}
   
	@Test
	public void testNaiveImport() throws Throwable {
		long start = System.currentTimeMillis();
		importLogic.naiveImport();
		long end = System.currentTimeMillis();
		log.info("Naive InnoDb import - time elapsed: " + (end - start) / 1000 + "s");
	}

	@Test
	public void testMultiRowImportLongChunks() throws Throwable {
		long start = System.currentTimeMillis();
		int chunkSize = LONG_CHUNK_SIZE;
		for (int i = 0; i < importLogic.getIterationCount() / chunkSize; i++) {
			importLogic.multiRowInsert(i + 1, chunkSize);
		}
		long end = System.currentTimeMillis();
		log.info("Long chunks multi row InnoDb import - time elapsed: " + (end - start) / 1000 + "s");
	}

	@Test
	public void testMultiRowImportShortChunks() throws Throwable {
		long start = System.currentTimeMillis();
		int chunkSize = SHORT_CHUNK_SIZE;
		for (int i = 0; i < importLogic.getIterationCount() / chunkSize; i++) {
			importLogic.multiRowInsert(i + 1, chunkSize);
		}
		long end = System.currentTimeMillis();
		log.info("Short chunks multi row InnoDb import - time elapsed: " + (end - start) / 1000 + "s");
	}

	@Test
	public void testMultiRowImportLongChunksUnderTransaction() throws Throwable {
		long start = System.currentTimeMillis();
		int chunkSize = LONG_CHUNK_SIZE;
		for (int i = 0; i < importLogic.getIterationCount() / chunkSize; i++) {
			importLogic.multiRowInsertUnderTransaction(i + 1, chunkSize);
		}
		long end = System.currentTimeMillis();
		log.info("Long chunks multi row in transaction InnoDb import - time elapsed: " + (end - start) / 1000 + "s");
	}

	@Test
	public void testMultiRowImportShortChunksUnderTransaction() throws Throwable {
		long start = System.currentTimeMillis();
		int chunkSize = SHORT_CHUNK_SIZE;
		for (int i = 0; i < importLogic.getIterationCount() / chunkSize; i++) {
			importLogic.multiRowInsertUnderTransaction(i + 1, chunkSize);
		}
		long end = System.currentTimeMillis();
		log.info("Short chunks multi row in transaction InnoDb import - time elapsed: " + (end - start) / 1000 + "s");
	}

	@Test
	public void testImportUnderTransaction() throws Throwable {
		long start = System.currentTimeMillis();
		importLogic.importUnderTransaction();
		long end = System.currentTimeMillis();
		log.info("Single huge transaction InnoDb import - time elapsed: " + (end - start) / 1000 + "s");
	}	

	@Test
	public void testImportUnderTransactionLongChunks() throws Throwable {
		long start = System.currentTimeMillis();
		int chunkSize = LONG_CHUNK_SIZE;
		for (int i = 0; i < importLogic.getIterationCount() / chunkSize; i++) {
			importLogic.importUnderTransactionPerPartes(i + 1, chunkSize);
		}
		long end = System.currentTimeMillis();
		log.info("Multiple large transactions InnoDb import - time elapsed: " + (end - start) / 1000 + "s");
	}

	@Test
	public void testImportUnderTransactionSmallChunks() throws Throwable {
		long start = System.currentTimeMillis();
		int chunkSize = SHORT_CHUNK_SIZE;
		for (int i = 0; i < importLogic.getIterationCount() / chunkSize; i++) {
			importLogic.importUnderTransactionPerPartes(i + 1, chunkSize);
		}
		long end = System.currentTimeMillis();
		log.info("Multiple small transactions InnoDb import - time elapsed: " + (end - start) / 1000 + "s");
	}

}
